﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Input;
using Windows.ApplicationModel;
using Windows.ApplicationModel.Core;
using Windows.UI.Core;
using KakiAISpeaker.Client.Helpers;
using Microsoft.Bot.Connector.DirectLine;
using Newtonsoft.Json;
using WebSocketSharp;

namespace KakiAISpeaker.Client.ViewModels
{
    /// <summary>
    ///     메인 페이지 뷰모델
    /// </summary>
    public class MainViewModel : BindableBase
    {
        /// <summary>
        ///     다이렉트 라인 시크릿
        /// </summary>
        private const string DIRECT_LINE_SECRET = "PZ0mGcox1m4.cwA.H-I.GGuo1qzJ6KCfE7QpmjNDw2EPk02xkhOlGKsd912iI48";

        private readonly string _botId = "KakiAISpeakerChannel";
        private readonly string _fromUser = "kaki104";

        /// <summary>
        ///     봇클라이언트
        /// </summary>
        private DirectLineClient _botClient;

        /// <summary>
        ///     컨버세이션
        /// </summary>
        private Conversation _conversation;

        private string _inputText;
        private string _watermark;

        private WebSocket _webSocketClient;

        /// <summary>
        ///     생성자
        /// </summary>
        public MainViewModel()
        {
            if (!DesignMode.DesignModeEnabled) Init();
        }

        /// <summary>
        ///     전송 버튼 커맨드
        /// </summary>
        public ICommand SendCommand { get; set; }

        /// <summary>
        ///     다이얼로그 목록
        /// </summary>
        public IList<Activity> Dialogs { get; set; }

        /// <summary>
        ///     입력받은 텍스트
        /// </summary>
        public string InputText
        {
            get => _inputText;
            set => Set(ref _inputText, value);
        }

        private async void Init()
        {
            Dialogs = new ObservableCollection<Activity>();
            SendCommand = new RelayCommand(ExecuteSendCommand);

            //봇 컨버세이션 시작
            await StartBotConversationAsync();
        }

        /// <summary>
        ///     봇 초기화
        /// </summary>
        /// <returns></returns>
        private async Task StartBotConversationAsync()
        {
            //다이렉트라인 클라이언트 생성
            _botClient = new DirectLineClient(DIRECT_LINE_SECRET);
            _conversation = await _botClient.Conversations.StartConversationAsync();

            Debug.WriteLine($"ConversationId : {_conversation.ConversationId}");

            //웹소켓 클라이언트
            _webSocketClient = new WebSocket(_conversation.StreamUrl);
            _webSocketClient.OnMessage += WebSocketClient_OnMessage;
            _webSocketClient.Connect();
        }

        /// <summary>
        ///     소켓 메시지 수신
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private async void WebSocketClient_OnMessage(object sender, MessageEventArgs e)
        {
            if (string.IsNullOrWhiteSpace(e.Data))
                return;

            var activitySet = JsonConvert.DeserializeObject<ActivitySet>(e.Data);
            //언젠가 쓰일지도 모르는..
            _watermark = activitySet.Watermark;

            //botId와 맞는 녀석만 골라내고..
            var activities = from x in activitySet.Activities
                where x.From.Id == _botId
                select x;

            foreach (var activity in activities)
            {
                await ExecuteDispatcherRunAsync(() => Dialogs.Add(activity));
            }
        }

        /// <summary>
        ///     전송 버튼 커맨드 실행
        /// </summary>
        private async void ExecuteSendCommand()
        {
            if (string.IsNullOrEmpty(InputText)) return;
            await SendMessageToBotAsync(InputText);
            InputText = string.Empty;
        }

        /// <summary>
        ///     메시지 송신
        /// </summary>
        /// <param name="inputText"></param>
        /// <returns></returns>
        private async Task SendMessageToBotAsync(string inputText)
        {
            var userMessage = new Activity
            {
                From = new ChannelAccount(_fromUser),
                Text = inputText,
                Type = ActivityTypes.Message
            };

            Dialogs.Add(userMessage);

            await _botClient.Conversations.PostActivityAsync(_conversation.ConversationId, userMessage);
        }

        /// <summary>
        ///     디스패처 실행
        /// </summary>
        /// <param name="handler"></param>
        /// <returns></returns>
        private async Task ExecuteDispatcherRunAsync(DispatchedHandler handler)
        {
            await CoreApplication.MainView.Dispatcher
                .RunAsync(CoreDispatcherPriority.Normal, handler);
        }
    }
}
